;;;; pkg-config.scm
;
; Copyright (c) 2022, Chris Brannon
; All rights reserved.
;
; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
; conditions are met:
;
;   Redistributions of source code must retain the above copyright notice, this list of conditions and the following
;     disclaimer. 
;   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following
;     disclaimer in the documentation and/or other materials provided with the distribution. 
;   Neither the name of the author nor the names of its contributors may be used to endorse or promote
;     products derived from this software without specific prior written permission. 
;
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
; OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
; AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR
; CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
; CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
; SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
; THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
; OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
; POSSIBILITY OF SUCH DAMAGE.
;
; Send bugs, suggestions and ideas to: 
;
; chris@the-brannons.com

(module pkg-config
    (pkg-config:cflags pkg-config:libs call-subprocess-for-output)
(import
  scheme
  (chicken base)
  (chicken io)
  (chicken process)
  (chicken string)
  posix-utils)

(define pkg-config-executable "pkg-config")

(define (call-subprocess-for-output program args)
  (receive
      (in out pid)
      (process program args)
    (close-output-port out)
    (let ((stdout-text (read-string #f in)))
      (receive (_ normal status)
	  (process-wait pid)
	(close-input-port in)
	(values (if (eof-object? stdout-text) "" stdout-text) status)))))

(define (call-pkg-config args)
  (receive (text status)
      (call-subprocess-for-output pkg-config-executable args)
    (if (zero? status) (string-split (string-trim-eol text)) #f)))

(define (pkg-config:cflags package)
    (call-pkg-config
     `("--cflags" ,package)))

(define (pkg-config:libs package #!key (static #f))
    (call-pkg-config
     (if static
	 `("--libs" "--static" ,package)
	 `("--libs" ,package))))

)
